require 'socket'

module Woody
  module Statsd
    class Client
      class << self
        def get(namespace = ENV['STATSD_NS'], host = ENV['STATSD_HOST'], port = ENV['STATSD_PORT'])
          @client ||= new(namespace, host, port)
        end
      end

      def initialize(namespace, host, port)
        @host = host || "localhost"
        @port = port || 8188
        @socket = UDPSocket.new
        @ns = namespace || "nanit"
      end

      def increment(stat, inc = 1)
        send_to_socket counter_stat(stat, inc)
      end

      def timing(stat, time_took)
        send_to_socket timing_stat(stat, time_took)
      end

      def with_timing(stat)
        start = Time.now.to_f
        res = yield
        time_took = (Time.now.to_f - start) * 1000
        timing(stat, time_took)
        res
      end

      def counter_stat(stat, inc)
        "#{@ns}.#{stat}:#{inc}|c\n"
      end

      def timing_stat(stat, time_took)
        "#{@ns}.#{stat}:#{time_took.round}|ms\n"
      end

      def send_to_socket(msg)
        begin
          @socket.send msg, 0, @host, @port
          nil
        rescue => e
          puts "WOODY_ERROR #{e.inspect} HOST #{@host} PORT #{@port}"
        end
      end
    end
  end
end
