require 'woody/statsd/client'
module Woody
  module Statsd
    module Sinatra
      class Middleware

        def initialize(app)
          @app = app
        end

        def call(env)
          start_time = Time.now
          status, headers, body  = @app.call(env)
          time_took = (1000 * (Time.now - start_time)).round
          if stat = env[Helpers::RACK_ENV_KEY]
            Reporter.report(stat, time_took, status)
          end
          [status, headers, body]
        end

        private

        class Reporter

          class << self
            def report(stat, time_took, status)
              new(stat, time_took, status).report
            end
          end

          def initialize(stat, time_took, status)
            @client = Woody::Statsd::Client.get
            @base_metric = base_metric(stat)
            @time = time_took
            @status = status
          end

          def report
            @client.increment "#{@base_metric}.requests"
            @client.increment "#{@base_metric}.response_codes.#{@status}"
            @client.timing "#{@base_metric}", @time
          end

          private

          def base_metric(stat)
            "web.#{stat}"
          end
        end
      end
    end
  end
end
